# coding: utf-8

import sys
import cv2
import numpy as np
from .util import *

nclr = 17 # len(pts) + 1
pts = [
    [0, 0], # 1/16
    [2, 2],
    [0, 2],
    [2, 0],
    [1, 1],
    [3, 3],
    [3, 1],
    [1, 3],
    [2, 3],
    [0, 1],
    [0, 3],
    [2, 1],
    [1, 0],
    [3, 2],
    [1, 2],
    [3, 1], # 16/16
]

'''
settings = [
    {'fc': 0, 'bc': 0, 'k': 0}, # b
    {'fc': 85, 'bc': 0, 'k': 1}, 
    {'fc': 85, 'bc': 0, 'k': 2}, 
    {'fc': 85, 'bc': 0, 'k': 3}, 
    {'fc': 85, 'bc': 0, 'k': 4}, 
    {'fc': 85, 'bc': 0, 'k': 5}, 
    {'fc': 85, 'bc': 0, 'k': 6}, 
    {'fc': 85, 'bc': 0, 'k': 7}, 
    {'fc': 85, 'bc': 0, 'k': 8}, 
    {'fc': 85, 'bc': 0, 'k': 9}, 
    {'fc': 85, 'bc': 0, 'k': 10}, 
    {'fc': 85, 'bc': 0, 'k': 11}, 
    {'fc': 85, 'bc': 0, 'k': 12}, 
    {'fc': 85, 'bc': 0, 'k': 13}, 
    {'fc': 85, 'bc': 0, 'k': 14}, 
    {'fc': 85, 'bc': 0, 'k': 15}, 
    {'fc': 0, 'bc': 85, 'k': 0}, # c1
    {'fc': 85, 'bc': 170, 'k': 15}, 
    {'fc': 85, 'bc': 170, 'k': 14}, 
    {'fc': 85, 'bc': 170, 'k': 13}, 
    {'fc': 85, 'bc': 170, 'k': 12}, 
    {'fc': 85, 'bc': 170, 'k': 11}, 
    {'fc': 85, 'bc': 170, 'k': 10}, 
    {'fc': 85, 'bc': 170, 'k': 9}, 
    {'fc': 85, 'bc': 170, 'k': 8}, 
    {'fc': 85, 'bc': 170, 'k': 7}, 
    {'fc': 85, 'bc': 170, 'k': 6}, 
    {'fc': 85, 'bc': 170, 'k': 5}, 
    {'fc': 85, 'bc': 170, 'k': 4}, 
    {'fc': 85, 'bc': 170, 'k': 3}, 
    {'fc': 85, 'bc': 170, 'k': 2}, 
    {'fc': 85, 'bc': 170, 'k': 1}, 
    {'fc': 0, 'bc': 170, 'k': 0}, # c2
    {'fc': 255, 'bc': 170, 'k': 1}, 
    {'fc': 255, 'bc': 170, 'k': 2}, 
    {'fc': 255, 'bc': 170, 'k': 3}, 
    {'fc': 255, 'bc': 170, 'k': 4}, 
    {'fc': 255, 'bc': 170, 'k': 5}, 
    {'fc': 255, 'bc': 170, 'k': 6}, 
    {'fc': 255, 'bc': 170, 'k': 7}, 
    {'fc': 255, 'bc': 170, 'k': 8}, 
    {'fc': 255, 'bc': 170, 'k': 9}, 
    {'fc': 255, 'bc': 170, 'k': 10}, 
    {'fc': 255, 'bc': 170, 'k': 11}, 
    {'fc': 255, 'bc': 170, 'k': 12}, 
    {'fc': 255, 'bc': 170, 'k': 13}, 
    {'fc': 255, 'bc': 170, 'k': 14}, 
    {'fc': 255, 'bc': 170, 'k': 15}, 
    {'fc': 0, 'bc': 255, 'k': 0}, # w
]
'''

'''
def make_noise(size, fc=0, bc=255, k=8):
    # P(fc) = k/16, P(bc) = (16-k)/16
    if k == 0: return np.zeros(size) + bc
    idx = np.random.random(size) < k/16
    img = np.where(idx, fc, bc)
    return img
'''

def make_grid(size, fc=0, bc=255, k=8):
    img = np.zeros(size) + bc
    for i in range(k):
        r, c = pts[i]
        img[r::4, c::4] = fc
    return img

def grid(img):
    assert img.ndim == 2

    patterns = [
        make_grid([4, 4], fc=255, bc=0, k=k) 
        for k in range(nclr)
    ]

    clrs = np.linspace(0, 255, nclr).astype(int)
    delims = (clrs[1:] + clrs[:-1]) // 2
    delims = np.asarray([0, *delims, 256])
    idcs = [
        np.where((img >= st) & (img < ed)) 
        for st, ed in zip(delims[:-1], delims[1:])
    ]
    
    img = img.copy()
    for idx, pt in zip(idcs, patterns):
        idxm4 = (idx[0] % 4, idx[1] % 4)
        img[idx] = pt[idxm4]
    
    return img

def grid_bts(img):
    if not is_img_data(img): return img
    img = conv2png(img)
    img = np.frombuffer(img, np.uint8)
    img = cv2.imdecode(img, cv2.IMREAD_GRAYSCALE)
    if img is None: return None
    img = grid(img).astype(np.uint8)
    img = cv2.imencode(
        '.png', img, 
        [cv2.IMWRITE_PNG_BILEVEL, 1]
    )[1]
    return bytes(img)

'''
def noise(img):
    assert img.ndim == 2

    clrs = np.linspace(0, 255, len(settings)).astype(int)
    delims = (clrs[1:] + clrs[:-1]) // 2
    delims = np.asarray([0, *delims, 256])
    idcs = [np.where((img >= st) & (img < ed)) for st, ed in zip(delims[:-1], delims[1:])]
    
    img = img.copy()
    for idx, kw in zip(idcs, settings):
        img[idx] = make_noise(len(idx[0]), **kw)
    
    return img

def noise_bts(img):
    if not is_img_data(img): return img
    img = conv2png(img)
    img = np.frombuffer(img, np.uint8)
    img = cv2.imdecode(img, cv2.IMREAD_GRAYSCALE)
    if img is None: return None
    img = noise(img).astype(np.uint8)
    img = cv2.imencode(
        '.png', img, 
        [cv2.IMWRITE_PNG_COMPRESSION, 9]
    )[1]
    return bytes(img)
'''

def noise(img):
    assert img.ndim == 2

    r = np.random.randint(255, size=img.shape)
    img = np.where(r < img, 255, 0)
    return img

def noise_bts(img):
    if not is_img_data(img): return img
    img = conv2png(img)
    img = np.frombuffer(img, np.uint8)
    img = cv2.imdecode(img, cv2.IMREAD_GRAYSCALE)
    if img is None: return None
    img = noise(img).astype(np.uint8)
    img = cv2.imencode(
        '.png', img, 
        [cv2.IMWRITE_PNG_BILEVEL, 1]
    )[1]
    return bytes(img)

noisebw = noise
noisebw_bts = noise_bts

def main():
    fname = sys.argv[1]
    img = open(fname, 'rb').read()
    img = grid_bts(img)
    with open(fname, 'wb') as f:
        f.write(img)

if __name__ == '__main__': main()


import argparse
import re
from os import path
from . import __version__
from .adathres import adathres_bts
from .dither import grid_bts, noise_bts, noisebw_bts
from .quant import pngquant_bts
from .trunc import trunc_bts
from multiprocessing import Pool
import copy
import traceback
from .util import *

modes = ['quant', 'grid', 'noise', 'noisebw', 'trunc', 'thres']

is_img = lambda s: re.search(r'\.(jpg|jpeg|gif|png|bmp|webp|tiff)$', s)

def process_file_safe(args):
    try:
        process_file(args)
    except:
        traceback.print_exc()
    
def process_file(args):
    if not is_img(args.fname):
        return
    print(args.fname)
    
    with open(args.fname, 'rb') as f:
        img = f.read()
    if not is_img_data(img):
        print('not valid img file')
        return
    
    if args.mode == 'grid':
        img = grid_bts(img)
    elif args.mode == 'noise':
        img = noise_bts(img)
    elif args.mode == 'noisebw':
        img = noisebw_bts(img)
    elif args.mode == 'trunc':
        img = trunc_bts(img, args.colors)
    elif args.mode == 'quant':
        img = pngquant_bts(img, args.colors)
    elif args.mode == 'thres':
        img = adathres_bts(img)
    if not img:
        print('fail to process')
        return
    
    fname = args.ofname or args.fname
    with open(fname, 'wb') as f:
        f.write(img)

def process_dir(args):
    dname = args.fname
    files = os.listdir(dname)
    
    pool = Pool(args.threads)
    for f in files:
        f = path.join(dname, f)
        args = copy.deepcopy(args)
        args.fname = f
        pool.apply_async(process_file_safe, [args])
    pool.close()
    pool.join()

def main():
    parser = argparse.ArgumentParser(prog="ImgYaso", description="provide various image compression methods", formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument("-v", "--version", action="version", version=f"ImgYaso version: {__version__}")
    parser.add_argument('fname', help='file name to be processed')
    parser.add_argument('-c', '--colors', type=int, default=8, help='num of colors')
    parser.add_argument('-m', '--mode', default=modes[0], choices=modes, help='processing mode')
    parser.add_argument('-o', '--ofname', help='output file name')
    parser.add_argument('-t', '--threads', type=int, default=4, help='num of threads')
    args = parser.parse_args()

    if not path.exists(args.fname):
        print('file not found')
        return
        
    if path.isdir(args.fname):
        process_dir(args)
    else:
        process_file(args)
        
    
if __name__ == '__main__': main()


import subprocess as subp
import tempfile
import uuid
import os
from os import path
from PIL import Image, ImageFile
from io import BytesIO

ImageFile.LOAD_TRUNCATED_IMAGES = True

headers = {
    'png': b'\x89PNG',
    # 'svg': b'<?xml',
    'jpg': b'\xff\xd8\xff',
    'gif': b'GIF',
    'tif': b'II\x2a\x00',
    'bmp': b'BM',
    'webp': b'RIFF',
}

def is_img_data(img):
    for _, hdr in headers.items():
        l = len(hdr)
        if img[:l] == hdr:
            return True
    return False

def conv2png(img):
    if img[:4] == headers['png']:
        return img
    img = Image.open(BytesIO(img))
    bio = BytesIO()
    img.convert('RGBA').save(bio, 'png')
    return bio.getvalue()



import numpy as np
from scipy import signal
import cv2
import re
import os
from os import path
import sys
from .util import *

def adathres_bts(img, win=9, beta=0.9):
    if not is_img_data(img): return img
    img = conv2png(img)
    img = np.frombuffer(img, np.uint8)
    img = cv2.imdecode(img, cv2.IMREAD_GRAYSCALE)
    if img is None: return None
    img = adathres(img, win, beta).astype(np.uint8)
    img = cv2.imencode(
        '.png', img, 
        [cv2.IMWRITE_PNG_BILEVEL, 1]
    )[1]
    return bytes(img)

def adathres(img, win=9, beta=0.9):
    if win % 2 == 0: win = win - 1
    # 边界的均值有点麻烦
    # 这里分别计算和和邻居数再相除
    kern = np.ones([win, win])
    sums = signal.correlate2d(img, kern, 'same')
    cnts = signal.correlate2d(np.ones_like(img), kern, 'same')
    means = sums // cnts
    # 如果直接采用均值作为阈值，背景会变花
    # 但是相邻背景颜色相差不大
    # 所以乘个系数把它们过滤掉
    img = np.where(img < means * beta, 0, 255)
    return img
    
def main():
    fname = sys.argv[1]
    img = open(fname, 'rb').read()
    img = adathres_bts(img)
    with open(fname, 'wb') as f:
        f.write(img)

if __name__ == '__main__': main()


import sys
import cv2
import numpy as np
from .util import *

def trunc_bts(img, l=4):
    if not is_img_data(img): return img
    img = conv2png(img)
    img = np.frombuffer(img, np.uint8)
    img = cv2.imdecode(img, cv2.IMREAD_GRAYSCALE)
    if img is None: return None
    img = trunc(img, l).astype(np.uint8)
    img = cv2.imencode(
        '.png', img, 
        [cv2.IMWRITE_PNG_COMPRESSION, 9]
    )[1]
    return bytes(img)


def trunc(img, l=4):
    assert img.ndim == 2
    
    colors = np.linspace(0, 255, l).astype(int)
    
    img_3d = np.expand_dims(img, 2)
    dist = np.abs(img_3d - colors)
    idx = np.argmin(dist, axis=2)
    img = colors[idx]
    
    return img

def main():
    fname = sys.argv[1]
    img = open(fname, 'rb').read()
    img = trunc_bts(img)
    with open(fname, 'wb') as f:
        f.write(img)
    
if __name__ == '__main__': main()


#!/usr/bin/env python3.7
# -*- coding: utf-8 -*-

import setuptools
import imgyaso

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("requirements.txt", "r", encoding="utf-8") as fh:
    install_requires = fh.read().splitlines()

setuptools.setup(
    name="imgyaso",
    version=imgyaso.__version__,
    url="https://github.com/apachecn/imgyaso",
    author=imgyaso.__author__,
    author_email=imgyaso.__email__,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: Other/Proprietary License",
        "Natural Language :: Chinese (Simplified)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Multimedia :: Graphics",
        "Topic :: Utilities",
    ],
    description="提供多种图片有损压缩方式，包括自适应二值化、网格仿色、颜色缩减",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=[
        "image",
        "processing",
        "lossy",
        "compression",
        "adaptive",
        "threshold",
        "bilevel",
        "grid",
        "dither",
        "quantize",
        "图像处理",
        "有损压缩",
        "自适应阈值",
        "二值化",
        "网格仿色",
        "颜色缩减",
    ],
    install_requires=install_requires,
    python_requires=">=3.6",
    entry_points={
        'console_scripts': [
            "imgyaso=imgyaso.__main__:main",
        ],
    },
    packages=setuptools.find_packages(),
)



# -*- coding: utf-8 -*-

"""ImgYaso
https://github.com/apachecn/imgyaso"""

from .adathres import adathres_bts, adathres
from .dither import grid_bts, grid, noise_bts, noise, noisebw_bts, noisebw
from .quant import pngquant_bts, pngquant
from .trunc import trunc_bts, trunc
from .util import *

__author__ = "ApacheCN"
__email__ = "apachecn@163.com"
__license__ = "SATA"
__version__ = "2023.1.27.3"



import sys
import cv2
import libimagequant as liq
from PIL import Image, ImageFile
from io import BytesIO
from .util import *

ImageFile.LOAD_TRUNCATED_IMAGES = True

def pngquant(img, ncolors=8):
    img = cv2.imencode(
        '.png', img, 
        [cv2.IMWRITE_PNG_COMPRESSION, 9]
    )[1]
    img = bytes(img)
    img = pngquant_bts(img, ncolors)
    img = np.frombuffer(img, np.uint8)
    return cv2.imdecode(img, cv2.IMREAD_UNCHANGED)

def pngquant_bts(img, ncolors=8):
    if not is_img_data(img): return img
    img = conv2png(img)
    img = Image.open(BytesIO(img)).convert('RGBA')
    w, h = img.width, img.height
    bytes = img.tobytes()
    
    attr = liq.Attr()
    attr.max_colors = ncolors
    img = attr.create_rgba(bytes, w, h, 0)
    res = img.quantize(attr)
    res.dithering_level = 1.0
    bytes = res.remap_image(img)
    palette = res.get_palette()
    
    img = Image.frombytes('P', (w, h), bytes)
    
    palette_data = []
    for color in palette:
        if color.a == 0:
            palette_data.extend([255, 255, 255])
        else:
            palette_data.extend([color.r, color.g, color.b])
    img.putpalette(palette_data)
    
    bio = BytesIO()
    img.save(bio, 'PNG', optimize=True)
    return bio.getvalue()

def main():
    fname = sys.argv[1]
    img = open(fname, 'rb').read()
    img = pngquant_bts(img)
    with open(fname, 'wb') as f:
        f.write(img)
    
if __name__ == '__main__': main()